# The name of the kmod package here.
%define kmod_name kpatch
%define module %{kmod_name}.ko

%define kernel_version 3.10.0-327.36.1.vz7.20.18
%define vzver %(echo '%{kernel_version}' | sed -e 's/^.*vz7\\.//')

# These defines are convenient to make the spec files for VZ7 and VL7 as
# close as possible.
%define kversion_base %{kernel_version}
%define kversion_dist %{kversion_base}
%define kversion %{kversion_dist}

%global _use_internal_dependency_generator 0

# Disable the building of the debug package(s).
%define debug_package %{nil}

# Do not strip files automatically, this would strip the module's signature
# as well
%define __strip /usr/bin/true

# The suffix of the subpackage's name.
%define subpkg %{vzver}

# The major release number. Increase it when rebuilding the package.
%define rel 2


Name:		%{kmod_name}-kmod
Version:	0.3.4
Release:	%{rel}.%{vzver}%{?dist}
# Epoch should be higher here than in the dist-vz7-uN branch the autobuilder
# uses. This makes sure the packages built from version-specific branches
# are preferred over those built automatically. No need to track the release
# numbers of the latter ones, etc.
Epoch:		1

Summary:	Dynamic kernel patching tools - the kernel-mode part

Group:		System Environment/Kernel
License:	GPLv2
URL:		https://github.com/dynup/kpatch

Source0:	https://github.com/dynup/kpatch/archive/v%{version}.tar.gz?/%{kmod_name}-%{version}.tar.gz

# Note. The patchset is the same as for kpatch packages. Some of the patches
# have nothing to do with the core kernel module but it is easier to maintain
# the packaging projects for kpatch and kpatch-kmod this way.

# Virtuozzo-specific patches.
Patch501:	vz001-kpatch-build-Support-Virtuozzo-Linux-as-well.patch
Patch502:	vz002-service-Use-kpatch-with-retries.patch
Patch503:	vz003-kmod-core-Do-not-taint-the-kernel.patch
Patch504:	vz004-kmod-core-added-a-minimal-fault-simulation-support.patch
Patch505:	vz005-Do-not-create-sysfs-files-for-the-patched-functions.patch
Patch506:	vz006-Revert-kpatch-deprecate-the-replace-command.patch
Patch507:	vz007-Do-not-mention-initrd-in-the-help-messages.patch
Patch508:	vz008-Added-logging-to-var-log-kpatch.patch
Patch509:	vz009-Implement-kpatch-uninstall-all.patch
Patch510:	vz010-Updated-the-Readme-file-to-better-reflect-the-specif.patch
Patch511:	vz011-kpatch-build-allow-making-Kpatch-compatible-patches-.patch
Patch512:	vz012-core-patch-handle-relocations-of-the-functions-with-.patch
Patch513:	vz013-Allow-for-extension-of-Kpatch-related-structures-in-.patch
Patch514:	vz014-build-look-for-create-diff-object-and-such-in-usr-li.patch
Patch515:	vz015-kpatch-do-not-assume-systemctl-is-present.patch

# The patches from upstream.
Patch1001:	0001-create-diff-object-Update-fixup-offsets-in-.rela__ex.patch
Patch1002:	0002-kpatch-build-fix-undefined-reference-to-kpatch_shado.patch
Patch1003:	0003-kpatch-build-fix-tree-wide-rebuild.patch
Patch1004:	0004-kpatch-build-fix-tree-wide-rebuild-on-RHEL-7.patch
Patch1005:	0005-kmod-core-fix-activeness-safety-checks-for-kernels-4.patch
Patch1006:	0006-create-diff-object-better-error-message-for-dup-file.patch
Patch1007:	0007-kpatch-build-Add-UTS_UBUNTU_RELEASE_ABI-symbol-for-u.patch
Patch1008:	0008-use-livepatch-4.5-features-in-Ubuntu-Xenial-kernel.patch
Patch1009:	0009-smatch-fix-kpatch_shadow_alloc-cleanup.patch
Patch1010:	0010-smatch-sparse-trivial-code-cleanups.patch
Patch1011:	0011-kpatch-build-make-patches-for-debug-kernels-too.patch

ExclusiveArch:	x86_64

BuildRequires:	vzkernel-devel = %{kernel_version}
BuildRequires:	perl
BuildRequires:	openssl
BuildRequires:	module-signing-keys

%description
This package contains %{module} kernel module, the kernel-mode part of
Kpatch. Kpatch is a Linux dynamic kernel patching tool which allows you
to patch a running kernel without rebooting or restarting any processes.

#===========================================================================

%package %{subpkg}

# The name of the package already includes vzversion, no need to
# keep it in the Release expression.
Release:	%{rel}%{?dist}
Summary:	Dynamic kernel patch manager - the kernel-mode part
Group:		System Environment/Kernel
License:	GPLv2

# Do not generate requirements automatically. By default the requirements
# like "kernel(__kmalloc) = abcd1234" are generated and tie the package to
# the corresponding kernel versions.
# kpatch-patch should pull the right kpatch-kmod package, so it is not
# needed to require the particular kernel explicitly. Such requirements
# proved to be problematic.
AutoReq: no

Provides:	kernel-modules >= %{kversion_dist}.%{_target_cpu}
# kpatch-build requires kpatch-kmod >= <...>
Provides:	%{kmod_name}-kmod = %{?epoch:%{epoch}:}%{version}-%{release}
# Just in case someone looks for kmod-kpatch-<...>:
Provides:	kmod-%{kmod_name}-%{kversion_base} = %{?epoch:%{epoch}:}%{version}-%{release}
# kpatch-patch requires kpatch-kmod-<kernel_version>
Provides:	%{kmod_name}-kmod-%{kversion_base} = %{?epoch:%{epoch}:}%{version}-%{release}

Requires(post):		/usr/sbin/depmod
Requires(post):		kmod
Requires(postun):	/usr/sbin/depmod
Requires(postun):	kmod

# The plugin will automatically remove unneeded kpatch-kmod* and
# readykernel-patch* packages.
Requires:	yum-plugin-readykernel

%description %{subpkg}
This package contains %{module} kernel module, the kernel-mode part of
Kpatch. Kpatch is a Linux dynamic kernel patching tool which allows you
to patch a running kernel without rebooting or restarting any processes.


%files %{subpkg}
%defattr(644,root,root,755)
%dir /lib/modules/%{kversion}
%dir /lib/modules/%{kversion}/extra
%dir /lib/modules/%{kversion}/extra/%{kmod_name}
/lib/modules/%{kversion}/extra/%{kmod_name}/*


%post %{subpkg}
echo "Running depmod..."
if [ -e "/boot/System.map-%{kversion}" ]; then
    /usr/sbin/depmod -aeF "/boot/System.map-%{kversion}" "%{kversion}" > /dev/null || :
fi
echo "Done."


%postun %{subpkg}
echo "Running depmod..."
if [ -e "/boot/System.map-%{kversion}" ]; then
    /usr/sbin/depmod -aeF "/boot/System.map-%{kversion}" "%{kversion}" > /dev/null || :
fi
echo "Done."

#===========================================================================

%prep
%setup -qn %{kmod_name}-%{version}

%patch501 -p1
%patch502 -p1
%patch503 -p1
%patch504 -p1
%patch505 -p1
%patch506 -p1
%patch507 -p1
%patch508 -p1
%patch509 -p1
%patch510 -p1
%patch511 -p1
%patch512 -p1
%patch513 -p1
%patch514 -p1
%patch515 -p1

%patch1001 -p1
%patch1002 -p1
%patch1003 -p1
%patch1004 -p1
%patch1005 -p1
%patch1006 -p1
%patch1007 -p1
%patch1008 -p1
%patch1009 -p1
%patch1010 -p1
%patch1011 -p1


%build
echo "Building Kpatch core module for the kernel %{kversion}."
make KPATCH_BUILD=/usr/src/kernels/%{kversion}/ -C kmod/core %{_smp_mflags}


%install
rm -rf %{buildroot}

install -d %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/
install -m 644 kmod/core/%{module} %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/
install -m 644 kmod/core/Module.symvers %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/

# Strip debug info from the module.
# Do not use _{__strip} macro here because it is redefined above.
/usr/bin/strip --strip-debug %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/%{module}

# Sign the module if the keys are available.
KEYPATH=%{_datadir}/module-signing-keys/kpatch/
MODSECKEY="${KEYPATH}/signing_key_kpatch.priv"
MODPUBKEY="${KEYPATH}/signing_key_kpatch.x509"
MODULE_FILE="%{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/%{module}"

if [[ -f "${MODSECKEY}" ]]; then
	/usr/src/kernels/%{kversion}/scripts/sign-file sha256 \
	  ${MODSECKEY} ${MODPUBKEY} ${MODULE_FILE}
	rm -f ${MODULE_FILE}.{sig,dig}

	if [ "~Module signature appended~" != "$(tail -c 28 ${MODULE_FILE})" ]; then
		echo "Module %{module} is still unsigned."
		exit 1
	fi
fi
